<?php

namespace Drupal\href_lang_exchange_href;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Access controller for the Site entity entity.
 *
 * @see \Drupal\href_lang_exchange_href\Entity\SiteEntity.
 */
class SiteEntityAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\href_lang_exchange_href\Entity\SiteEntityInterface $entity */

    switch ($operation) {

      case 'view':

        if (!$entity->isPublished()) {
          return AccessResult::allowedIfHasPermission($account, 'view unpublished site entity entities');
        }

        return AccessResult::allowedIfHasPermission($account, 'view published site entity entities');

      case 'update':
      case 'edit':
        return AccessResult::allowedIfHasPermission($account, 'edit site entity entities');

      case 'delete':

        return AccessResult::allowedIfHasPermission($account, 'delete site entity entities');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add site entity entities');
  }

}
